function fn2re(s)
{
	return s.replace(/\.\+\$\{\}\=\!\<\>\|\:\//g, "\\$&").replace(/\*/g, ".*").replace(/\?/g, ".");
}


function makeCategoryRe(s)
{
	try { return new RegExp("(^|\\W)(" + fn2re(getValue("category")).replace(/ +/gm, "|") + ")($|\\W)"); } catch (e) { return null; }
}


function makeFnmatchRe(s)
{
	try { return new RegExp("^" + fn2re(s) + "$", "i"); } catch (e) { return null; }
}


function validatePage(node)
{
	var errors = [];
	// if id attribute is not found, throw exception
	switch (node.getAttribute("id"))
	{
		case "criteria":
			if (getValue("numItems").match(/\D/)) errors.push(localize("numItemsInvalid"));
			if (getValue("dateS") != "" && new Date(getValue("dateS")) == "Invalid Date") errors.push(localize("dateSInvalid"));
			if (getValue("dateE") != "" && new Date(getValue("dateE")) == "Invalid Date") errors.push(localize("dateEInvalid"));
			if (getValue("category") != "" && makeCategoryRe(getValue("category")) == null) errors.push(localize("categoryInvalid"));
			if (getValue("guid") != "" && makeFnmatchRe(getValue("guid")) == null) errors.push(localize("guidInvalid"));
			break;

		case "pubdate":
			if (getValue("channelPubDate") != "" && new Date(getValue("channelPubDate")) == "Invalid Date") errors.push(localize("pubDateInvalid"));
			break;
	}

	if (errors.length)
	{
		for (var i=0, count=errors.length, errStr=""; i<count; i++)
			errStr += errors[i] + ((i < count-1) ? "\n" : "");
		openMsgBox(errStr, 48);
		return false;
	}
	else
		return true;
}


function refreshData()
{
	const srcDocument = window.opener.document;
	const channel = document.getElementById("channel");
	var i, count;

	count = channel.childNodes.length;
	for (i=0; i<count; i++)
		switch (channel.childNodes[i].nodeName)
		{
			case "textbox":
				channel.childNodes[i].value = srcDocument.getElementById(channel.childNodes[i].getAttribute("id")).value;
				break;
			case "checkbox":
				channel.childNodes[i].checked = srcDocument.getElementById(channel.childNodes[i].getAttribute("id")).checked;
				break;
		}

	setValue("channelPubDate", boolPref("publishPubDateAuto") ? time() : srcDocument.getElementById("channelPubDate"));


	const srcList = window.opener.document.getElementById("itemList");
	const list = document.getElementById("itemList");
	var selection = [];

	var maxSelect = parseInt(getValue("numItems"));
	var dateS = getValue("dateS") == "" ? null : new Date(getValue("dateS"));
	var dateE = getValue("dateE") == "" ? null : new Date(getValue("dateE"));
	var category = getValue("category") == "" ? null : makeCategoryRe(getValue("category"));
	var guid = getValue("guid") == "" ? null : makeFnmatchRe(getValue("guid"));
	var pubDate, item;

	while (list.lastChild)
		list.removeChild(list.lastChild);

	// check the list according to criteria
	count = srcList.childNodes.length;
	for (i=0; i<count; i++)
	{
		item = eval(srcList.childNodes[i].firstChild.lastChild.getAttribute("label"));
		// if no pubDate, the criteria is not satisfied
		pubDate = new Date(item.pubDate);
		if ((dateS == null || dateS <= pubDate) && (dateE == null || dateE >= pubDate) && (category == null || category.test(item.category)) && (guid == null || guid.test(item.guid)))
		{
			selection.push({date: pubDate, index: i});
		}
	}

	// sort the selection by reverse chronological order, and then output the list
	selection.sort(function comp(a, b) {return b.date - a.date;});
	// if selection is longer than maximum, just ignore all the rest
	count = maxSelect > 0 && selection.length > maxSelect? maxSelect : selection.length;
	for (i=0; i<count; i++)
		list.appendChild(srcList.childNodes[selection[i].index].cloneNode(true));
}


function init()
{
	window.sdi = {baseTitle: document.documentElement.getAttribute("title")}; // set sdi.baseTitle for display error dialogs
	window.channelData = window.opener.channelData; // copy channelData from opener
	window.moveToCenterPosition();
	// Temporary fix for _hitEnter bug in wizard binding
	document.documentElement._hitEnter = function()
	{
		// if a button is focused, dispatch its command instead of advancing the wizard
		var focused = document.commandDispatcher.focusedElement;
		if (!(focused && focused.localName == "button"))
		this.advance();
	};
}


// fake function called by saveRSS in rssio.js
function itemUpdate() {}


// Store persistances
function unload()
{
	var i, a;
	for (i in a = ["numItems", "dateS", "dateE", "category", "guid"])
		document.getElementById(a[i]).setAttribute("value", getValue(a[i]));
}


// Publish to file
function publishFile()
{
	var file = new rssFilePicker().save(localize("Save"));
	if (!file) return;
	document.getElementById("saveMsg").setAttribute("value", localize(save(file, false) ? "SaveFileSuccess" : "SaveFileFailure"));
}


// Publish to URI
function publishURI()
{
	var arg = {title: localize("SaveURI")};
	window.openDialog("uripicker.xul", "_blank", "modal", arg);
	if (arg.value != true) return;
	var resume = function(aStatus)
	{
		if (aStatus)
		{
			openMsgBox(localize("SaveURIFailure", aStatus), 16, sdi.baseTitle);
			document.getElementById("saveMsg").setAttribute("value", localize("SaveURIFailure", aStatus));
		}
		else
			document.getElementById("saveMsg").setAttribute("value", localize("SaveURISuccess"));
	};
	if (saveToURI(arg.uri, false, resume))
		document.getElementById("saveMsg").setAttribute("value", localize("SavingToURI"));
	else
		document.getElementById("saveMsg").setAttribute("value", "");
}